# Copyright (c) 2007-2008 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: tests.py 5639 2008-03-27 09:59:57Z nilo $

import os
import unittest

from zope.testing import doctest

import zope.app.testing.functional


Layer = zope.app.testing.functional.ZCMLLayer(
    os.path.join(os.path.dirname(__file__), 'ftesting.zcml'),
    __name__, 'gocept.form.layer', allow_teardown=True)


def FunctionalDocFileSuite(*paths, **kw):
    try:
        layer = kw['layer']
    except KeyError:
        layer = Layer
    else:
        del kw['layer']
    kw['package'] = doctest._normalize_module(kw.get('package'))
    test = zope.app.testing.functional.FunctionalDocFileSuite(
        *paths, **kw)
    test.layer = layer
    return test


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(FunctionalDocFileSuite(
        'base.txt', 'grouped.txt', 'destructive-action.txt', 
        'multiple-constraints.txt', 'jsvalidation.txt'))
    return suite



# Copyright (c) 2007-2008 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: jsvalidation.py 5532 2008-01-24 13:39:39Z zagy $

import xml.sax.saxutils

import zope.contentprovider.interfaces
import zope.component
import zope.formlib.interfaces
import zope.interface
import zope.schema.interfaces
import zope.i18n

import zc.resourcelibrary

import gocept.mochikit.interfaces

import gocept.form.interfaces
from gocept.form.i18n import MessageFactory as _


class ValidationViewlet(zope.viewlet.viewlet.ViewletBase):

    def __init__(self, context, request, view, manager):
        super(ValidationViewlet, self).__init__(
            context, request, view, manager)
        self.validators = []

    def update(self):
        for form_field in self.__parent__.form_fields:
            field = form_field.field
            validators = zope.component.getAdapters(
                (field, self.request),
                gocept.form.interfaces.IJSValidator)
            self.validators.extend(validators)

    def render(self):
        result = ['<script language="javascript">']
        for name, validator in self.validators:
            result.append('// %s' % name)
            result.append(
                'connect(window, "onload", function(event) {%s});' % validator)
        result.append('</script>')
        return '\n'.join(result)


@zope.component.adapter(
    zope.schema.interfaces.IMinMaxLen,
    gocept.form.interfaces.IJSValidationLayer)
@zope.interface.implementer(gocept.form.interfaces.IJSValidator)
def max_length_validator(field, request):
    """validate the maximum length is not overridden"""
    if not field.max_length:
        return
    zc.resourcelibrary.need('gocept.form.jsvalidation')
    message = _("Too long (max ${max_length})",
               mapping=dict(max_length=field.max_length))
    return '''new gocept.validation.MaxLength('form.%s', %d, %s)'''  % (
        field.__name__, field.max_length,
        xml.sax.saxutils.quoteattr(
            zope.i18n.translate(message, context=request)))



# Copyright (c) 2007 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: base.py 5639 2008-03-27 09:59:57Z nilo $

import zope.app.container.interfaces

def apply_data_with_setattr(context, form_fields, data, adapters=None):
    """Applies data of a form to an object using setattr. Allows to adapt
    form fields to given interfaces."""

    if adapters is None:
        adapters = {}

    changed = False

    for form_field in form_fields:
        field = form_field.field

        name = form_field.__name__
        newvalue = data.get(name, form_field)  # using form_field as marker
        if newvalue is form_field:
            continue

        # Adapt context, if necessary
        interface = form_field.interface
        adapter = adapters.get(interface)
        if adapter is None:
            if interface is None:
                adapter = context
            else:
                adapter = interface(context)
            adapters[interface] = adapter

        setattr(adapter, name, newvalue)

    return changed


class Add(object):

    factory = None
    redirect_to_context_after_add = False

    def applyChanges(self, object, data):
        return apply_data_with_setattr(
            object, self.form_fields, data)

    def create(self, data):
        obj = self.factory()
        self.applyChanges(obj, data)
        return obj

    def add(self, object):
        chooser = zope.app.container.interfaces.INameChooser(self.context)
        name = chooser.chooseName(self.get_name(object), object)
        self.context[name] = object
        self._added_object = self.context[name]
        self._finished_add = True

    def nextURL(self):
        if self.redirect_to_context_after_add:
            target = self.context
        else:
            target = self._added_object

        return zope.component.getMultiAdapter(
            (target, self.request), name='absolute_url')()

    def get_name(self, object):
        return object.__class__.__name__



class Edit(object):

    redirect_to_parent_after_edit = True
    redirect_to_view = None
    new_context_interface = None

    def render(self):
        next_url = self.nextURL()
        if not self.errors and self.status and next_url is not None:
            self.notify(self.status)
            self.request.response.redirect(next_url)
            return ''
        return super(Edit, self).render()

    def notify(self, message):
        """hook to notify the user"""
        pass

    def nextURL(self):
        if (not self.redirect_to_parent_after_edit
            and not self.redirect_to_view
            and not self.new_context_interface):
            return None

        new_context = self.new_context()
        if self.redirect_to_parent_after_edit:
            new_context = new_context.__parent__

        view = ''
        if self.redirect_to_view:
            view = '/@@%s' % self.redirect_to_view

        return '%s%s' % (
            zope.component.getMultiAdapter(
                (new_context, self.request), name='absolute_url')(),
            view)

    def new_context(self):
        if self.new_context_interface:
            return self.new_context_interface(self.context)
        return self.context



# Copyright (c) 2007-2008 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: schema.py 5531 2008-01-24 13:37:55Z zagy $

import zope.schema.interfaces


def applySchemaData(context, schema, data, omit=()):
    """Apply `data` to `context` using `schema`."""
    omit = set(omit)
    for name in schema:
        if name in omit:
            continue
        field = schema[name]
        if not zope.schema.interfaces.IField.providedBy(field):
            continue
        value = data.get(name, field.default)
        setattr(context, name, value)



# Copyright (c) 2007-2008 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: __init__.py 5531 2008-01-24 13:37:55Z zagy $

#namespace package boilerplate
try:
    __import__('pkg_resources').declare_namespace(__name__)
except ImportError, e:
    from pkgutil import extend_path
    __path__ = extend_path(__path__, __name__)




# -*- coding: latin-1 -*-
# Copyright (c) 2007-2008 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: action.py 5532 2008-01-24 13:39:39Z zagy $
"""Additional actions."""

import zope.formlib.form
import zope.formlib.namedtemplate

class DestructiveAction(zope.formlib.form.Action):
    pass


@zope.formlib.namedtemplate.implementation(DestructiveAction)
def render_destructive_action(self):
    button = zope.formlib.form.render_submit_button(self)()
    if button == '':
        return ''

    # Extend the button with a check box that is synchronized with the
    # 'active' status of the button.

    additional = """
        <script type="text/javascript">
        document.getElementById('%s').disabled = true;
        </script>

        <label>
            <input
            type="checkbox" 
            id="%s" 
            onChange="document.getElementById('%s').disabled = !this.checked"/> Unlock &quot;%s&quot; button</label>

    """ % (self.__name__, self.__name__ + '.unlock', self.__name__, self.label)
    # XXX: the label should be translated.

    return button + additional


class destructive_action(zope.formlib.form.action):
    """Decorator to create destructive actions in form views."""

    def __call__(self, success):
        action = DestructiveAction(self.label, success=success, **self.options)
        self.actions.append(action)
        return action



# Copyright (c) 2007-2008 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: grouped.py 5531 2008-01-24 13:37:55Z zagy $

import zope.formlib.namedtemplate
import zope.formlib.interfaces
import zope.formlib.form

import zope.app.pagetemplate

import gocept.form.interfaces


grouped_form_template = zope.formlib.namedtemplate.NamedTemplateImplementation(
    zope.app.pagetemplate.ViewPageTemplateFile('grouped-form.pt'),
    zope.formlib.interfaces.IPageForm)


class Fields(object):

    zope.interface.implements(gocept.form.interfaces.IFieldGroup)

    def __init__(self, title, fields, css_class=None):
        self.title = title
        self.fields = fields
        self.css_class = css_class

    def get_field_names(self):
        return self.fields


class RemainingFields(object):

    zope.interface.implements(gocept.form.interfaces.IRemainingFields)

    def __init__(self, title, css_class=None):
        self.title = title
        self.css_class = css_class

    def get_field_names(self):
        return None


class FormBase(object):
    """Baseclass for forms."""

    template = zope.formlib.namedtemplate.NamedTemplate('gocept.form.grouped')
    field_groups = None
    title = u''

    def setUpWidgets(self, ignore_request=False):
        self.adapters = {}
        self.widgets = None
        remainder_group = None
        fields = []
        if self.field_groups is None:
            self.field_groups = []

        for group in self.field_groups:

            if gocept.form.interfaces.IRemainingFields.providedBy(group):
                widgets = None
                remainder_group = group
            else:
                field_names = group.get_field_names()
                # Select only form fields that actually exist
                form_field_names = [x.__name__ for x in self.form_fields]
                field_names = [x for x in field_names if x in
                               form_field_names]
                widgets = self._get_widgets(self.form_fields.select(
                    *field_names), ignore_request)
                if self.widgets is None:
                    self.widgets = widgets
                else:
                    self.widgets += widgets
                fields.extend(field_names)

            group.widgets = widgets

        # we create a default widget_group which puts all the rest of the
        # fields in one group
        remaining_fields = self.form_fields.omit(*fields)
        if remaining_fields:
            if remainder_group is None:
                remainder_group = RemainingFields(u'')
                self.field_groups = tuple(self.field_groups) + (
                    remainder_group,)
            widgets = self._get_widgets(remaining_fields, ignore_request)
            remainder_group.widgets = widgets
            if self.widgets is None:
                self.widgets = widgets
            else:
                self.widgets += widgets


class Form(FormBase, zope.formlib.form.Form):
    """Edit form base."""

    def _get_widgets(self, form_fields, ignore_request):
        return zope.formlib.form.setUpWidgets(
            form_fields, self.prefix, self.context, self.request,
            adapters=self.adapters, ignore_request=ignore_request)


class AddForm(FormBase, zope.formlib.form.AddForm):
    """Add form base."""

    def _get_widgets(self, form_fields, ignore_request):
        return zope.formlib.form.setUpInputWidgets(
            form_fields, self.prefix, self.context, self.request,
            ignore_request=ignore_request)

class EditForm(FormBase, zope.formlib.form.EditForm):
    """Edit form base."""

    def _get_widgets(self, form_fields, ignore_request):
        return zope.formlib.form.setUpEditWidgets(
            form_fields, self.prefix, self.context, self.request,
            adapters=self.adapters, ignore_request=ignore_request)


class DisplayForm(FormBase, zope.formlib.form.PageDisplayForm):
    """Display form base."""

    def _get_widgets(self, form_fields, ignore_request):
        return zope.formlib.form.setUpEditWidgets(
            form_fields, self.prefix, self.context, self.request,
            adapters=self.adapters, for_display=True,
            ignore_request=ignore_request)



# Copyright (c) 2007-2008 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: i18n.py 5532 2008-01-24 13:39:39Z zagy $

import zope.i18nmessageid


MessageFactory = zope.i18nmessageid.MessageFactory('gocept.form')
